/*
	Sudoku generator

	Batch sudoku generator

	(C) Copyright 2005 by Mogens Kjaer, mk@lemo.dk

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#include "sudoku.h"

int nfilled=28;
int bestndigits=0;
int ndigits;
int symmetrical=1;
int level=LEVEL_EASY;
int thislevel=0;
char symbollist[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

int statistics[MAXNXNY*MAXNXNY][NLEVELS];
static time_t start_time, stop_time;

int displayboard[MAXNXNY][MAXNXNY];

int setboard[MAXNXNY][MAXNXNY];
int setboardsave[MAXNXNY][MAXNXNY];
int beforeboard[MAXNXNY][MAXNXNY];
int board[MAXNXNY][MAXNXNY];
int nx=3;
int ny=3;
int nxny=9;
int nsol=0;
int ntest=0;
int running=0;
int stop_pressed=0;
int letter=0;
int print_explain=1;
int solved=0;
int solution_unique=0;

int nlines=0;
char *loglines=NULL;
int logsize=0;

void loginit()
{
  if(loglines!=NULL) free(loglines);
  loglines=NULL;
  logsize=0;
  nlines=0;
}

void logend()
{
}

void logprintf(const char *template, ...)
{
  va_list ap;
  char tmpbuf[1024];
  int len,i;

  va_start(ap, template);
  vsprintf(tmpbuf, template, ap);
  va_end(ap);
  len=strlen(tmpbuf);
  for(i=0;i<len;i++)
  {
    if(tmpbuf[i]=='\n') nlines++;
  }
  
  if(loglines==NULL)
  {
    loglines=malloc(len+1);
    loglines[0] = '\0';
  }
  else
  {
    loglines=realloc(loglines, strlen(loglines)+len+1);
  }
  strcat(loglines, tmpbuf);
}

void setcreatemenutext(char *txt)
{
}

void setsolvemenutext(char *txt)
{
}

void settitle(char *txt)
{
}

int test_stop()
{
  return 0;
}

void main_update()
{
  int i,j;
  printf("insert into sudoku values(NULL,now(),%d,%d,%d,%d,%d,0,'",nx,ny,bestndigits,thislevel,symmetrical);
  for(i=0;i<nxny;i++)
  for(j=0;j<nxny;j++) printf("%c", symbollist[displayboard[i][j]]);
  printf("','");
  solve_sudoku(0);
  for(i=0;i<nxny;i++)
  for(j=0;j<nxny;j++) printf("%c", symbollist[abs(displayboard[i][j])]);
  printf("');\n");
  fflush(stdout);
}

main(int argc, char **argv)
{
  int isudoku, nsudoku;
  char *env;
  unsigned int seed;

  env=getenv("SUDOKU_SEED");
  if(env)
  {
    seed=atoi(env);
  }
  else
  {
    seed=time(NULL);
  }

  srandom(seed);

  if(argc>1)
  {
    nsudoku=atoi(argv[1]);
    if(argc>2)
    {
      nx=atoi(argv[2]);
      if(argc>3)
      {
        ny=atoi(argv[3]);
	if(argc>4)
	{
	  symmetrical=atoi(argv[4]);
	}
	else
	{
	  symmetrical=1;
	}
      }
      else
      {
        ny=3;
      }
    }
    else
    {
      nx=3;
    }
  }
  else
  {
    nsudoku=100;
  }
  if(nx<2) nx=2;
  if(ny<2) ny=2;
  if(nx>MAXNX) nx=MAXNX;
  if(ny>MAXNY) ny=MAXNY;
  if(symmetrical<0) symmetrical=0;
  if(symmetrical>=NSYMMETRICALS) symmetrical=NSYMMETRICALS-1;
  nxny=nx*ny;

  init_bitmaps(nxny);

  nfilled=1;
  level = LEVEL_ANY;

  running = 1;
  for(isudoku=0;isudoku<nsudoku;isudoku++)
  {
    bestndigits=0;
    sudoku_run();
  }

}
